/* vim: set textwidth=80 tabstop=4: */

/* Copyright (c) 2008 Michael Rasmussen (mir@datanom.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __CALDAV_H__
#define __CALDAV_H__

#include <time.h>

/* For debug purposes */
struct debug_curl {
  char trace_ascii; /* 0 or 1 */
  char debug;       /* 0 or 1 */
};

typedef struct _caldav_error caldav_error;
struct _caldav_error {
	long code; /* if < 0 means an internal error. Otherwise an error related
				  to the CalDAV protocol. */
	char* str; /* For storing human readable error message */
};

/* CalDAV is defined in RFC4791 */

/* Buffer to hold response */
typedef struct _response response;
struct _response {
	char* msg;
};

/* enum specifying supported CalDAV actions.
 * UNKNOWN. An unknown action.
 * ADD. Add a CalDAV calendar object.
 * DELETE. Delete a CalDAV calendar object.
 * MODIFY. Modify a CalDAV calendar object.
 * GET. Get one or more CalDAV calendar object(s).
 * GETALL. Get all CalDAV calendar objects.
 */
typedef enum {
	UNKNOWN,
	ADD,
	DELETE,
	MODIFY,
	GET,
	GETALL,
	GETCALNAME,
	ISCALDAV
} CALDAV_ACTION;

/* enum specifying CalDAV error states.
 * OK (HTTP 200). Request was satisfied.
 * FORBIDDEN (HTTP 403). Access not allowed. Dont repeat request.
 * CONFLICT (HTTP 409). Conflict between current state of CalDAV collection
 * and request. Client must solve the conflict and then resend request.
 */
typedef enum {
	OK,
	FORBIDDEN,
	CONFLICT
} CALDAV_RESPONSE;

/* object: Appointment following ICal format (RFC2445). Receiver is responsible
 * for freeing the memory.
 * start, end: time_t variable specifying start and end for range. Both
 * are included in range.
 * URL: Defines CalDAV resource. Receiver is responsible for freeing the memory.
 * [http://][username[:password]@]host[:port]/url-path. See (RFC1738).
 * response: Address to the struct _response where the response is to be
 * stored.
 */
CALDAV_RESPONSE caldav_add_object(const char* object, const char* URL);
CALDAV_RESPONSE caldav_delete_object(const char* object, const char* URL);
CALDAV_RESPONSE caldav_modify_object(const char* object, const char* URL);
CALDAV_RESPONSE caldav_get_object(
	response* result, time_t start, time_t end, const char* URL);
CALDAV_RESPONSE caldav_getall_object(response* result, const char* URL);
CALDAV_RESPONSE caldav_get_displayname(response* result, const char* URL);

/* Function to test wether a calendar resource is CalDAV enabled or not.
 * URL: Defines CalDAV resource. Receiver is responsible for freeing the memory.
 * [http://][username[:password]@]host[:port]/url-path. See (RFC1738).
 * response: 0 (zero) means no CalDAV support, otherwise CalDAV support
 * detechted.
 */
int caldav_enabled_resource(const char* URL);

/* Function to call in case of errors.
 * Caller provides a pointer to a local caldav_error structure.
 * Caldav_get_error will initialize pointer if NULL.
 * Caller is responsible for freeing returned memory.
 * After the first call the internal error buffer is reset.
 */
caldav_error* caldav_get_error(caldav_error* lib_error);
void caldav_free_error(caldav_error* lib_error);

/* Setting various options in library */
void caldav_set_options(struct debug_curl curl_options);

#endif
